# -*- coding: utf-8 -*-
import asyncio
import logging

from playwright.async_api import async_playwright

from cf_clearance import async_cf_retry, async_stealth
from tests.test_common import (
    test_add_ua_cookie_cf_success,
    test_not_pass_cf_challenge_request,
)


async def test_cf_challenge():
    async with async_playwright() as p:
        browser = await p.chromium.launch(
            headless=False,
            proxy={"server": "socks5://localhost:7890"},
            args=[
                "--disable-gpu",
                "--no-sandbox",
                "--disable-dev-shm-usage",
                "--no-first-run",
                "--no-service-autorun",
                "--no-default-browser-check",
                "--password-store=basic",
                "--start-maximized",
            ],
        )
        content = await browser.new_context(no_viewport=True)
        page = await content.new_page()
        await async_stealth(page, pure=True)
        await page.goto("https://nowsecure.nl")
        res = await async_cf_retry(page)
        ua = None
        cf_clearance_value = None
        if res:
            cookies = await page.context.cookies()
            for cookie in cookies:
                if cookie.get("name") == "cf_clearance":
                    cf_clearance_value = cookie.get("value")
            ua = await page.evaluate("() => {return navigator.userAgent}")
        else:
            logging.info("cf challenge fail")
        await browser.close()
        return ua, cf_clearance_value


async def main():
    logging.basicConfig(level=logging.INFO)
    test_not_pass_cf_challenge_request()
    ua, cf_clearance_value = await test_cf_challenge()
    test_add_ua_cookie_cf_success(ua, cf_clearance_value)


asyncio.get_event_loop().run_until_complete(main())



# -*- coding: utf-8 -*-
from playwright.async_api import Error
from playwright.async_api import Page as AsyncPage
from playwright.sync_api import Page as SyncPage


async def async_cf_retry(page: AsyncPage, tries: int = 10) -> bool:
    success = False
    while tries > 0:
        await page.wait_for_timeout(1500)
        try:
            success = False if await page.query_selector("#challenge-form") else True
            if success:
                break
            simple_challenge = await page.query_selector(
                "#challenge-stage > div > input[type='button']"
            )
            if simple_challenge:
                await simple_challenge.click()
            for target_frame in page.main_frame.child_frames:
                if "challenge" in target_frame.url and "turnstile" in target_frame.url:
                    click = await target_frame.query_selector(
                        "xpath=//input[@type='checkbox']"
                    )
                    if click:
                        await click.click()
        except Error:
            success = False
        tries -= 1
    return success


def sync_cf_retry(page: SyncPage, tries: int = 10) -> bool:
    success = False
    while tries > 0:
        page.wait_for_timeout(1500)
        try:
            success = False if page.query_selector("#challenge-form") else True
            if success:
                break
            simple_challenge = page.query_selector(
                "#challenge-stage > div > input[type='button']"
            )
            if simple_challenge:
                simple_challenge.click()
            for target_frame in page.main_frame.child_frames:
                if "challenge" in target_frame.url and "turnstile" in target_frame.url:
                    click = target_frame.query_selector(
                        "xpath=//input[@type='checkbox']"
                    )
                    if click:
                        click.click()
        except Error:
            success = False
        tries -= 1

    return success



from cf_clearance.retry import async_cf_retry, sync_cf_retry
from cf_clearance.stealth import StealthConfig, async_stealth, sync_stealth

__version__ = "0.29.2"

__all__ = (
    "async_stealth",
    "sync_stealth",
    "async_cf_retry",
    "sync_cf_retry",
    "StealthConfig",
)



import logging

import requests


def test_not_pass_cf_challenge_request():
    proxies = {"all": "socks5://localhost:7890"}
    res = requests.get("https://nowsecure.nl", proxies=proxies)
    if "<title>Just a moment...</title>" in res.text:
        logging.info("not use clearance cf challenge fail")


def test_add_ua_cookie_cf_success(ua, cf_clearance):
    proxies = {"all": "socks5://localhost:7890"}
    headers = {"user-agent": ua}
    cookies = {"cf_clearance": cf_clearance}

    res = requests.get(
        "https://nowsecure.nl", proxies=proxies, headers=headers, cookies=cookies
    )
    if "<title>Just a moment...</title>" not in res.text:
        logging.info("use clearance cf challenge success")



# -*- coding: utf-8 -*-
import re
from dataclasses import dataclass
from typing import Dict, Union

import pkg_resources
from playwright.async_api import BrowserContext as AsyncContext
from playwright.async_api import Page as AsyncPage
from playwright.sync_api import BrowserContext as SyncContext
from playwright.sync_api import Page as SyncPage


def from_file(name):
    """Read script from ./js directory"""
    return pkg_resources.resource_string("cf_clearance", f"js/{name}").decode()


SCRIPTS: Dict[str, str] = {
    "chrome_fp": from_file("canvas.fingerprinting.js"),
    "chrome_global": from_file("chrome.global.js"),
    "chrome_touch": from_file("emulate.touch.js"),
    "navigator_permissions": from_file("navigator.permissions.js"),
    "navigator_webdriver": from_file("navigator.webdriver.js"),
    "chrome_runtime": from_file("chrome.runtime.js"),
    "chrome_plugin": from_file("chrome.plugin.js"),
}


@dataclass
class StealthConfig:
    """
    Playwright stealth configuration that applies stealth strategies to playwright page objects.
    The stealth strategies are contained in ./js package and are basic javascript scripts that are executed
    on every page.goto() called.
    Note:
        All init scripts are combined by playwright into one script and then executed this means
        the scripts should not have conflicting constants/variables etc. !
        This also means scripts can be extended by overriding enabled_scripts generator:
        ```
        @property
        def enabled_scripts():
            yield 'console.log("first script")'
            yield from super().enabled_scripts()
            yield 'console.log("last script")'
        ```
    """

    # load script options
    chrome_fp: bool = True
    chrome_global: bool = True
    chrome_touch: bool = True
    navigator_permissions: bool = True
    navigator_webdriver: bool = True
    chrome_runtime: bool = True
    chrome_plugin: bool = True

    @property
    def enabled_scripts(self):

        if self.chrome_fp:
            yield SCRIPTS["chrome_fp"]
        if self.chrome_global:
            yield SCRIPTS["chrome_global"]
        if self.chrome_touch:
            yield SCRIPTS["chrome_touch"]
        if self.navigator_permissions:
            yield SCRIPTS["navigator_permissions"]
        if self.navigator_webdriver:
            yield SCRIPTS["navigator_webdriver"]
        if self.chrome_plugin:
            yield SCRIPTS["chrome_plugin"]
        if self.chrome_runtime:
            yield SCRIPTS["chrome_runtime"]


def sync_stealth(
    page_or_context: Union[SyncContext, SyncPage],
    config: StealthConfig = None,
    pure: bool = True,
):
    """teaches synchronous playwright Page to be stealthy like a ninja!"""
    for script in (config or StealthConfig()).enabled_scripts:
        page_or_context.add_init_script(script)
    if pure:
        page_or_context.route(
            re.compile(
                r"(.*\.png(\?.*|$))|(.*\.jpg(\?.*|$))|(.*\.jpeg(\?.*|$))|(.*\.css(\?.*|$))"
            ),
            lambda route: route.abort("blockedbyclient"),
        )


async def async_stealth(
    page_or_context: Union[AsyncContext, AsyncPage],
    config: StealthConfig = None,
    pure: bool = True,
):
    """teaches asynchronous playwright Page to be stealthy like a ninja!"""
    for script in (config or StealthConfig()).enabled_scripts:
        await page_or_context.add_init_script(script)
    if pure:
        await page_or_context.route(
            re.compile(
                r"(.*\.png(\?.*|$))|(.*\.jpg(\?.*|$))|(.*\.jpeg(\?.*|$))|(.*\.css(\?.*|$))"
            ),
            lambda route: route.abort("blockedbyclient"),
        )



from playwright.async_api import async_playwright

from cf_clearance import async_cf_retry, async_stealth


async def test_cf_challenge(url: str):
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=False)
        context = await browser.new_context()
        page = await context.new_page()
        await async_stealth(page, pure=True)
        await page.goto(url)
        res = await async_cf_retry(page)
        if res:
            cookies = await page.context.cookies()
            for cookie in cookies:
                if cookie.get("name") == "cf_clearance":
                    cf_clearance_value = cookie.get("value")
        assert cf_clearance_value
        await browser.close()


if __name__ == "__main__":
    import asyncio

    asyncio.run(test_cf_challenge("https://nowsecure.nl"))
